# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"check_if_port_is_bound": "init.ipynb",
         "get_random_port_numbers": "init.ipynb",
         "logger": "init.ipynb",
         "generate_port_pairings": "init.ipynb",
         "lookup_host": "init.ipynb",
         "lookup_host_port": "init.ipynb",
         "lookup_socket_name": "init.ipynb",
         "NamedPorts": "init.ipynb",
         "set_port_name": "init.ipynb",
         "block_for_startup": "init.ipynb",
         "block_for_command": "init.ipynb",
         "run_container": "init.ipynb",
         "start": "redis.ipynb"}

modules = ["__init__.py",
           "redis.py"]

doc_url = "https://talosiot.github.io/scylla/"

git_url = "https://github.com/talosiot/scylla/tree/master/"

def custom_doc_links(name): return None
